AGN = {};
AGN.commands = {};
AGN.player = {};
AGN.player.guild = {};
AGN.frame = CreateFrame("Frame", "AGN");

AGN_CONFIG = {};
AGN_CONFIG.debug = false;
AGN_CONFIG.enabled = true;

function AGN:HasGuildInfo() return self.player.guild.rankOrder ~= nil; end

function AGN:TryUpdateNote()
    if not self:HasGuildInfo() then return; end

    self.frame:RegisterEvent("INSPECT_READY");
    NotifyInspect("player");
end

function AGN:d(msg)
    if AGN_CONFIG.debug ~= true then return; end
    print("[DEBUG] " .. msg);
end

function AGN.frame:OnEvent(event, ...)
    local handler = self[event];
    if (handler == nil) then return; end

    handler(self, event, ...);
end

function AGN.frame:ADDON_LOADED(_, addonName)
    if addonName ~= "AutoGuildNote" then return; end
end

function AGN.frame:PLAYER_LOGIN(_)
    self:RegisterEvent("GUILD_ROSTER_UPDATE");

    local playerName, playerRealm = UnitFullName("player");
    AGN.player.name = playerName;
    AGN.player.realm = playerRealm;
    AGN.player.fullName = playerName .. "-" .. playerRealm;
    AGN.player.guid = UnitGUID("player");

    -- local guildName, guildRankName, guildRankIndex, guildRealm =
    --     GetGuildInfo("player");
    -- AGN.player.guild.name = guildName;
    -- AGN.player.guild.realm = guildRealm;
    -- AGN.player.guild.fullName = guildName .. "-" .. guildRealm;
    -- AGN.player.guild.rankIndex = guildRankIndex;
    -- AGN.player.guild.rankName = guildRankName;

    AGN:d("AGN loaded");
    AGN:d("player.guid = " .. AGN.player.guid);

    C_GuildInfo.GuildRoster();
    AGN:TryUpdateNote();
end

function AGN.frame:GUILD_ROSTER_UPDATE(_)
    AGN.player.guild.rankOrder = C_GuildInfo.GetGuildRankOrder(AGN.player.guid);
    AGN.player.guild.rankName = GuildControlGetRankName(
                                    AGN.player.guild.rankOrder);
    -- don't set note for alt ranks
    if (string.find(string.lower(AGN.player.guild.rankName), "alt") ~= nil) then
        AGN:d("Not setting guild note for rank " .. AGN.player.guild.rankName);
        self:UnregisterEvent("GUILD_ROSTER_UPDATE");
        return;
    end

    AGN:TryUpdateNote();
end

function AGN.frame:INSPECT_READY(_, targetGuid)
    if not AGN_CONFIG.enabled then return; end
    if not AGN:HasGuildInfo() then
        AGN:d("Not ready to update note");
        return;
    end

    AGN:d("targetGuid = " .. targetGuid);
    if (targetGuid ~= AGN.player.guid) then return; end

    local honorLevel = UnitHonorLevel("player");
    local itemLevel = C_PaperDollInfo.GetInspectItemLevel("player");
    local versPercent = GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE);

    local topBracket = nil;
    local topRating = 0;

    local rating2v2 = GetInspectArenaData(1);
    if (rating2v2 > topRating) then
        topBracket = "2v2";
        topRating = rating2v2;
    end

    local rating3v3 = GetInspectArenaData(2);
    if (rating3v3 > topRating) then
        topBracket = "3v3";
        topRating = rating3v3;
    end

    local ratingRbg = GetInspectArenaData(4);
    if (ratingRbg > topRating) then
        topBracket = "rbg";
        topRating = ratingRbg;
    end

    local note = "HL" .. honorLevel .. "/i" .. itemLevel;
    if topRating > 0 then
        note = note .. "/CR" .. topRating .. " (" .. topBracket .. ")";
    end
    note = note .. "/v" .. string.format("%i", versPercent) .. "%";

    print("Setting own public note to \"" .. note .. "\"");

    for i = 1, GetNumGuildMembers() do
        if GetGuildRosterInfo(i) == AGN.player.fullName then
            GuildRosterSetPublicNote(i, note);
            self:UnregisterEvent("GUILD_ROSTER_UPDATE");
            break
        end
    end

    ClearInspectPlayer();
end

AGN.frame:RegisterEvent("ADDON_LOADED");
AGN.frame:RegisterEvent("PLAYER_LOGIN");
AGN.frame:SetScript("OnEvent", AGN.frame.OnEvent);
